@@ -5,6 +5,7 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
+     Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
 Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
@@ -40,6 +41,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include "main.h"
 
 #include "fits.h"
+#include "meta.h"
 #include "keywords.h"
 
 
@@ -101,7 +103,7 @@ fits_print_extension_info(struct fitsparams *p)
   /* Open the FITS file and read the first extension type, upon moving to
      the next extension, we will read its type, so for the first we will
      need to do it explicitly. */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
   if (fits_get_hdu_type(fptr, &hdutype, &status) )
     gal_fits_io_error(status, "reading first extension");
 
@@ -325,7 +327,7 @@ fits_hdu_number(struct fitsparams *p)
   int numhdu, status=0;
 
   /* Read the first extension (necessary for reading the rest). */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
 
   /* Get the number of HDUs. */
   if( fits_get_num_hdus(fptr, &numhdu, &status) )
@@ -352,13 +354,12 @@ fits_datasum(struct fitsparams *p)
 
 
 
-static void
-fits_pixelscale(struct fitsparams *p)
+struct wcsprm *
+fits_read_check_wcs(struct fitsparams *p, size_t *ndim,
+                    char *operation)
 {
   int nwcs=0;
-  size_t i, ndim=0;
   struct wcsprm *wcs;
-  double multip, *pixelscale;
 
   /* Read the desired WCS. */
   wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
@@ -366,11 +367,26 @@ fits_pixelscale(struct fitsparams *p)
 
   /* If a WCS doesn't exist, let the user know and return. */
   if(wcs)
-    ndim=wcs->naxis;
+    *ndim=wcs->naxis;
   else
     error(EXIT_FAILURE, 0, "%s (hdu %s): no WCS could be read by WCSLIB, "
-          "hence the pixel-scale cannot be determined", p->input->v,
-          p->cp.hdu);
+          "hence the %s cannot be determined", p->input->v,
+          p->cp.hdu, operation);
+
+  /* Return the WCS. */
+  return wcs;
+}
+
+
+
+
+
+static void
+fits_pixelscale(struct fitsparams *p)
+{
+  size_t i, ndim=0;
+  double multip, *pixelscale;
+  struct wcsprm *wcs=fits_read_check_wcs(p, &ndim, "pixel scale");
 
   /* Calculate the pixel-scale in each dimension. */
   pixelscale=gal_wcs_pixel_scale(wcs);
@@ -473,6 +489,42 @@ fits_pixelscale(struct fitsparams *p)
 
 
 
+static void
+fits_pixelarea(struct fitsparams *p)
+{
+  double area;
+  size_t i, ndim=0;
+  double *pixelscale;
+  struct wcsprm *wcs=fits_read_check_wcs(p, &ndim, "pixel area");
+
+  /* Basic sanity checks. */
+  if(ndim!=2)
+    error(EXIT_FAILURE, 0, "the pixel area can only be calculated "
+          "for two dimensional data (images). The given input has "
+          "%zu dimensions", ndim);
+  for(i=0;i<ndim;++i)
+    if( strcmp(wcs->cunit[i], "deg") )
+      error(EXIT_FAILURE, 0, "%s (hdu: %s): dimension %zu doesn't "
+            "have a unit of degrees (value to 'CUNIT%zu'), but "
+            "'%s'! In order to use '--pixelareaarcsec2', the units "
+            "of the WCS have to be in degrees", p->input->v,
+            p->cp.hdu, i+1, i+1, wcs->cunit[i]);
+
+  /* Calculate the pixel-scale in each dimension. */
+  pixelscale=gal_wcs_pixel_scale(wcs);
+
+  /* Calcluate the pixel area (currently only in arcsec^N) */
+  area=1;
+  for(i=0;i<ndim;++i) area *= pixelscale[i]*3600;
+  printf("%g\n", area);
+
+  /* Clean up. */
+  wcsfree(wcs);
+  free(pixelscale);
+}
+
+
+
 static void
 fits_skycoverage(struct fitsparams *p)
 {
@@ -708,7 +760,7 @@ fits_hdu_remove(struct fitsparams *p, int *r)
       hdu=gal_list_str_pop(&p->remove);
 
       /* Open the FITS file at the specified HDU. */
-      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE);
+      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE, 1);
 
       /* Delete the extension. */
       if( fits_delete_hdu(fptr, &hdutype, &status) )
@@ -771,7 +823,7 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 
       /* Open the FITS file at the specified HDU. */
       in=gal_fits_hdu_open(p->input->v, hdu,
-                           cut1_copy0 ? READWRITE : READONLY);
+                           cut1_copy0 ? READWRITE : READONLY, 1);
 
       /* If the output isn't opened yet, open it.  */
       if(out==NULL)
@@ -813,24 +865,27 @@ fits(struct fitsparams *p)
 
   switch(p->mode)
     {
+    /* Functions creating meta-images, in other words, images conveying
+       information about the given image. */
+    case FITS_MODE_META: meta_pixelareaonwcs(p); break;
+
     /* Keywords, we have a separate set of functions in 'keywords.c'. */
-    case FITS_MODE_KEY:
-      r=keywords(p);
-      break;
+    case FITS_MODE_KEY: r=keywords(p); break;
 
     /* HDU, functions defined here. */
     case FITS_MODE_HDU:
 
       /* Options that must be called alone. */
-      if(p->numhdus) fits_hdu_number(p);
-      else if(p->datasum)       fits_datasum(p);
-      else if(p->pixelscale)    fits_pixelscale(p);
-      else if(p->skycoverage)   fits_skycoverage(p);
-      else if(p->hasimagehdu)   fits_certain_hdu(p, 0, 0);
-      else if(p->hastablehdu)   fits_certain_hdu(p, 0, 1);
-      else if(p->listimagehdus) fits_certain_hdu(p, 1, 0);
-      else if(p->listtablehdus) fits_certain_hdu(p, 1, 1);
-      else if(p->listallhdus)   fits_list_all_hdus(p);
+      if(p->numhdus)               fits_hdu_number(p);
+      else if(p->datasum)          fits_datasum(p);
+      else if(p->pixelscale)       fits_pixelscale(p);
+      else if(p->pixelareaarcsec2) fits_pixelarea(p);
+      else if(p->skycoverage)      fits_skycoverage(p);
+      else if(p->hasimagehdu)      fits_certain_hdu(p, 0, 0);
+      else if(p->hastablehdu)      fits_certain_hdu(p, 0, 1);
+      else if(p->listimagehdus)    fits_certain_hdu(p, 1, 0);
+      else if(p->listtablehdus)    fits_certain_hdu(p, 1, 1);
+      else if(p->listallhdus)      fits_list_all_hdus(p);
 
       /* Options that can be called together. */
       else
