@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -53,6 +53,33 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /****************************************************************
  ************           Initial detection            ************
  ****************************************************************/
+/* The user gives connectivity values in terms of the number of neighbors
+   to use, so with this function, we convert that number to the
+   connectivity that is defined in `gnuastro/dimension.h'. */
+static int
+detection_ngb_to_connectivity(size_t ndim, size_t ngb)
+{
+  int connectivity=0;
+  switch(ndim)
+    {
+    case 2:
+      connectivity = ngb==4 ? 1 : 2;
+      break;
+    case 3:
+      connectivity = ngb==6 ? 1 : (ngb==18 ? 2 : 3);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
+            "the problem. %zu not a valid value to `ndim'",
+            __func__, PACKAGE_BUGREPORT, ndim);
+    }
+  return connectivity;
+}
+
+
+
+
+
 void
 detection_initial(struct noisechiselparams *p)
 {
@@ -92,7 +119,9 @@ detection_initial(struct noisechiselparams *p)
 
   /* Erode the image. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  gal_binary_erode(p->binary, p->erode, p->erodengb==4 ? 1 : 2, 1);
+  gal_binary_erode(p->binary, p->erode,
+                   detection_ngb_to_connectivity(p->input->ndim,
+                                                 p->erodengb), 1);
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "Eroded %zu time%s (%zu-connected).", p->erode,
@@ -116,7 +145,9 @@ detection_initial(struct noisechiselparams *p)
 
   /* Do the opening. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  gal_binary_open(p->binary, p->opening, p->openingngb==4 ? 1 : 2, 1);
+  gal_binary_open(p->binary, p->opening,
+                  detection_ngb_to_connectivity(p->input->ndim,
+                                                p->openingngb), 1);
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "Opened (depth: %zu, %zu-connected).",
@@ -141,7 +172,7 @@ detection_initial(struct noisechiselparams *p)
                                                     p->binary->ndim);
   if(p->detectionname)
     {
-      p->olabel->name="OPENED_AND_LABELED";
+      p->olabel->name="OPENED-AND-LABELED";
       gal_fits_img_write(p->olabel, p->detectionname, NULL, PROGRAM_NAME);
       p->olabel->name=NULL;
     }
@@ -283,7 +314,9 @@ detection_fill_holes_open(void *in_prm)
 
       /* Fill the holes in this tile: holes with maximal connectivity means
          that they are most strongly bounded. */
-      gal_binary_holes_fill(copy, p->holengb==4 ? 1 : 2, -1);
+      gal_binary_holes_fill(copy, detection_ngb_to_connectivity(p->input->ndim,
+                                                                p->holengb),
+                            -1);
       if(fho_prm->step==1)
         {
           detection_write_in_large(tile, copy);
@@ -293,7 +326,9 @@ detection_fill_holes_open(void *in_prm)
         }
 
       /* Open all the regions. */
-      gal_binary_open(copy, p->dopening, p->dopeningngb==4 ? 1 : 2, 1);
+      gal_binary_open(copy, p->dopening,
+                      detection_ngb_to_connectivity(p->input->ndim,
+                                                    p->dopeningngb), 1);
 
       /* Write the copied region back into the large input and AFTERWARDS,
          correct the tile's pointers, the pointers must not be corrected
@@ -327,8 +362,8 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
   float *f;
   uint8_t *b, *bf;
   gal_data_t *bin;
-  int con=p->pseudoconcomp==4 ? 1 : 2;
   struct fho_params fho_prm={0, NULL, workbin, worklab, p};
+  int con=detection_ngb_to_connectivity(p->input->ndim, p->pseudoconcomp);
 
 
   /* Set all the initial detected pixels to blank values. */
@@ -502,10 +537,10 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   size_t tablen=num+1;
   gal_data_t *sn, *snind;
   int32_t *plabend, *indarr=NULL;
-  double ave, err, *xy, *brightness;
+  double ave, err, *pos, *brightness;
+  size_t ind, ndim=p->input->ndim, pcols=1+ndim;
   float s, ss, *f, *ff, *fs, *sky=p->sky->array;
-  size_t ind, ndim=p->input->ndim, xyncols=1+ndim;
-  size_t i, *area, counter=0, *dsize=p->input->dsize;
+  size_t i, j, *area, counter=0, *dsize=p->input->dsize;
   int32_t *plab = worklab->array, *dlab = s0d1D2 ? NULL : p->olabel->array;
   size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
                                       "coord");
@@ -518,9 +553,9 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
     error(EXIT_FAILURE, 0, "%s: only a NaN value is recognized for blank "
           "floating point data types, the blank value is defined to be %f",
           __func__, GAL_BLANK_FLOAT32);
-  if(ndim!=2)
-    error(EXIT_FAILURE, 0, "%s: only 2D datasets are acceptable, your input "
-          "is %zu dimensions", __func__, ndim);
+  if(ndim!=2 && ndim!=3)
+    error(EXIT_FAILURE, 0, "%s: only 2D images or 3D datacubes are "
+          "acceptable, but input has %zu dimensions", __func__, ndim);
 
 
   /* Allocate all the necessary arrays, note that since we want to put each
@@ -530,8 +565,8 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
                                     "area");
   brightness = gal_pointer_allocate(GAL_TYPE_FLOAT64, tablen, 1, __func__,
                                      "brightness");
-  xy         = gal_pointer_allocate(GAL_TYPE_FLOAT64, xyncols*tablen, 1,
-                                     __func__, "xy");
+  pos        = gal_pointer_allocate(GAL_TYPE_FLOAT64, pcols*tablen, 1,
+                                    __func__, "pos");
   flag       = ( s0d1D2==0
                  ? gal_pointer_allocate(GAL_TYPE_UINT8, tablen, 1, __func__,
                                          "flag")
@@ -575,9 +610,11 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           brightness[*plab] += ss;
           if( ss > 0.0f )  /* For calculatiing the approximate center, */
             {              /* necessary for calculating Sky and STD.   */
-              xy[*plab*xyncols  ] += ss;
-              xy[*plab*xyncols+1] += (double)coord[0] * ss;
-              xy[*plab*xyncols+2] += (double)coord[1] * ss;
+              pos[  *plab*pcols   ] += ss;
+              pos[  *plab*pcols+1 ] += (double)coord[0] * ss;
+              pos[  *plab*pcols+2 ] += (double)coord[1] * ss;
+              if(ndim==3)
+                pos[*plab*pcols+3 ] += (double)coord[2] * ss;
             }
         }
 
@@ -589,13 +626,14 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
 
 
   /* A small sanity check.
-  {
-    size_t i;
-    for(i=1;i<num+1;++i)
-      printf("%zu (%u): %-5zu %-13.3f %-13.3f %-13.3f %-13.3f\n", i, flag[i],
-             area[i], brightness[i], xy[i*xyncols], xy[i*xyncols+1],
-             xy[i*xyncols+2]);
-  }
+  if(s0d1D2==1)
+    {
+      size_t i;
+      for(i=1;i<num+1;++i)
+        printf("%zu (%u): %-5zu %-13.3f %-13.3f %-13.3f %-13.3f\n",
+               i, dlab ? flag[i] : 255, area[i], brightness[i],
+               pos[i*pcols], pos[i*pcols+1], pos[i*pcols+2]);
+    }
   */
 
 
@@ -626,11 +664,11 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   for(i=1;i<tablen;++i)
     {
       ave=brightness[i]/area[i];
-      if( area[i]>p->snminarea && ave>0.0f && xy[i*xyncols]>0.0f )
+      if( area[i]>=p->snminarea && ave>0.0f && pos[i*pcols]>0.0f )
         {
           /* Get the flux weighted center coordinates. */
-          coord[0]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+1]/xy[i*xyncols] );
-          coord[1]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+2]/xy[i*xyncols] );
+          for(j=0;j<ndim;++j)
+            coord[j]=GAL_DIMENSION_FLT_TO_INT(pos[i*pcols+j+1]/pos[i*pcols]);
 
           /* Get the Sky standard deviation on this tile. */
           err  = ((float *)(p->std->array))[
@@ -677,7 +715,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
 
 
   /* Clean up and return. */
-  free(xy);
+  free(pos);
   free(area);
   free(coord);
   free(brightness);
