@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+     Pedram Ashofteh Ardakani <pedramardakani@pm.me>
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -57,6 +58,13 @@ gsl_rng *
 gal_checkset_gsl_rng(uint8_t envseed_bool, const char **name,
                      unsigned long int *seed);
 
+size_t
+gal_checkset_ram_available(int quietmmap);
+
+int
+gal_checkset_need_mmap(size_t bytesize, size_t minmapsize, int quietmmap);
+
+
 
 
 /**************************************************************/
@@ -86,6 +94,8 @@ gal_checkset_allocate_copy_set(char *arg, char **copy, int *set);
 char *
 gal_checkset_dataset_name(char *filename, char *hdu);
 
+char *
+gal_checkset_timestamp(char *filename, char *newext);
 
 
 
@@ -98,6 +108,12 @@ gal_checkset_dir_part(char *filename);
 char *
 gal_checkset_not_dir_part(char *filename);
 
+char *
+gal_checkset_suffix_separate(char *name, char **suffix);
+
+char *
+gal_checkset_make_unique_suffix(char *reference, char *suffix);
+
 void
 gal_checkset_check_file(char *filename);
 
