@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -36,6 +36,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/units.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/tableintern.h>
@@ -121,6 +122,30 @@ gal_txt_trim_space(char *str)
 
 
 
+/* Return 1 if the input string contains the 'match' substring. */
+int
+gal_txt_contains_string(char *full, char *match)
+{
+  char *c=full;
+  size_t ml = match ? strlen(match) : 0;
+
+  /* If the input or output strings are empty, a match can't be defined
+     (strncmp will always succeed to find a match). */
+  if(ml==0 || full==NULL || *full=='\0') return 0;
+
+  /* Parse the string (character by character) to see if there is a
+     match. */
+  do if( strncmp(c, match, ml)==0 ) return 1;
+  while( *(++c)!='\0' );
+
+  /* If control reaches here, there was no match. */
+  return 0;
+}
+
+
+
+
+
 /* Each information comment should have a format like this (replace
    'Column' with 'Image' for 2D arrays):
 
@@ -245,12 +270,11 @@ txt_info_from_comment(char *in_line, gal_data_t **datall, char *comm_start,
 
 
       /* If 'typestr' was given, then check if this is a standard type. If
-         'typestr' wasn't specified, then the default double type code will
-         be used (see the variable definitions above). If the given type
-         isn't a standard type then ignore the line. Just note that if we
-         are dealing with the string type, we have to pull out the number
-         part first. If there is no number for a string type, then ignore
-         the line. */
+         'typestr' wasn't specified or couldn't be interpretted, then the
+         default double type code will be used (see the variable
+         definitions above). Just note that if we are dealing with the
+         string type, we have to pull out the number part first. If there
+         is no number for a string type, then ignore the line. */
       if(typestr && *typestr!='\0')
         {
           typestr=gal_txt_trim_space(typestr);
@@ -263,7 +287,8 @@ txt_info_from_comment(char *in_line, gal_data_t **datall, char *comm_start,
           else
             {
               type=gal_type_from_name(typestr);
-              if(type==GAL_TYPE_INVALID) return;
+              if(type==GAL_TYPE_INVALID)
+                type=GAL_TYPE_FLOAT64;
             }
         }
 
@@ -315,8 +340,10 @@ static size_t
 txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
                         int inplace)
 {
+  double tmpd;
+  void *tmpdptr=&tmpd;
   gal_data_t *col, *prev, *tmp;
-  size_t n=0, maxcnum=0, numtokens;
+  size_t ncol=0, maxcnum=0, numtokens;
   char *line, *token, *end, *aline=NULL;
 
   /* Make a copy of the input line if necessary. */
@@ -348,18 +375,24 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
     maxcnum = maxcnum>col->status ? maxcnum : col->status;
 
   /* Go over the line check/fill the column information. */
-  while(++n)
+  while(++ncol)
     {
+      /* If 'line' has already passed the end of the actual string (for
+         example when a string column is the last one and its declared
+         width is larger than the actual number of characters it has in
+         that line). */
+      if(line>=end) break;
+
       /* When we are dealing with a text table, check if there is
          information for this column. For a text image, only the number of
          tokens is important (as the second dimension of the image), so
          just assume there no information. */
       if(format==TXT_FORMAT_TABLE)
-        for(col=*datall; col!=NULL; col=col->next) {if(col->status==n) break;}
+        for(col=*datall; col!=NULL; col=col->next)
+          { if(col->status==ncol) break; }
       else
         col=NULL;
 
-
       /* If there is information for this column, then check if it is a
          string, and if so, don't use 'strtok_r' (because it might have
          delimiters). So manually go ahead in the line till you get to the
@@ -375,30 +408,30 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
               /* Increment line to the end of the string. */
               line = (token=line) + col->disp_width;
 
-              /* If we haven't reached the end of the line, then set a NULL
-                 character where the string ends, so we can use the
-                 token. VERY IMPORTANT: this should not be '<=end'. If the
-                 given width is larger than line, there is no problem, the
-                 '\0' of the line will also be used to end this last
-                 column.*/
+              /* If we haven't reached the end of the line (so 'line<end'),
+                 then set a NULL character where the current token is
+                 expected to end. In this way, we can use the token (while
+                 preserving the line for the rest of the 'while'
+                 loop). VERY IMPORTANT: to do this, 'line' should not be
+                 '<=end'. If the given width is larger than line, there is
+                 no problem, the '\0' of the line will also be used to end
+                 this last column.*/
               if(line<end)
                 {
                   *line++='\0';
                   /* printf(" col %zu: -%s-\n", i, token); */
                 }
-              else break;
             }
           else
             {
-              token=strtok_r(n==1?line:NULL, GAL_TXT_DELIMITERS, &line);
+              token=strtok_r(ncol==1?line:NULL, GAL_TXT_DELIMITERS, &line);
               if(token==NULL) break;
-              /* printf(" col %zu: =%s=\n", i, token); */
             }
         }
       else
         {
           /* Make sure a token exists in this undefined column. */
-          token=strtok_r(n==1?line:NULL, GAL_TXT_DELIMITERS, &line);
+          token=strtok_r(ncol==1?line:NULL, GAL_TXT_DELIMITERS, &line);
           if(token==NULL) break;
 
           /* A token exists. For a table, define a new element in the
@@ -410,14 +443,28 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
              information). */
           if( *datall==NULL || format==TXT_FORMAT_TABLE )
             {
+              /* Make sure the token is actually a number (or RA/Dec
+                 written in Sexagesimal format) and print a good error
+                 message when the input isn't actually a number but a
+                 string (this test was added because of uncommented
+                 metadata)! */
+              if( gal_type_from_string( &tmpdptr, token, GAL_TYPE_FLOAT64)
+                  && isnan( gal_units_ra_to_degree(token) )
+                  && isnan( gal_units_dec_to_degree(token) ) )
+                error(EXIT_FAILURE, 0, "'%s' couldn't be read as a "
+                      "number (element %zu of first uncommented line)",
+                      token, ncol);
+
+              /* Allocate this column's dataset and set it's 'status' to
+                 the column number that it corresponds to. */
               gal_list_data_add_alloc(datall, NULL, GAL_TYPE_FLOAT64, 0,
-                                      NULL, NULL, 0, -1, 1, NULL, NULL, NULL);
-              (*datall)->status=n;
+                                      NULL, NULL, 0, -1, 1, NULL, NULL,
+                                      NULL);
+              (*datall)->status=ncol;
             }
         }
     }
 
-
   /* When looking at a text table, 'n' is the number of columns (elements
      in the linked list). But when looking at an image, it is the size of
      the second dimension. To unify things from this step forwards, we will
@@ -428,20 +475,21 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
 
      Note that 'n' counts from 1, so the total number of tokens is one less
      than 'n'.*/
-  numtokens=n-1;
-  if(format==TXT_FORMAT_IMAGE) n=1;
+  numtokens=ncol-1;
+  if(format==TXT_FORMAT_IMAGE) ncol=1;
 
   /* If the number of columns/images given by the comments is larger than
      the actual number of lines, remove those that have larger numbers from
      the linked list before things get complicated outside of this
      function. */
-  if(maxcnum>n)
+  if(maxcnum>numtokens)
     {
       prev=NULL;
       col=*datall;
       while(col!=NULL)
         {
-          if(col->status > n) /* Column has no data (was only in comments) */
+          /* This column has no data (was only in comments) */
+          if(col->status > numtokens)
             {
               /* This column has to be removed/freed. But we have to make
                  some corrections before freeing it:
@@ -461,7 +509,7 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
               gal_data_free(col);
               col=tmp;
             }
-          else                /* Column has data.                          */
+          else /* Column has data. */
             {
               prev=col;
               col=col->next;
@@ -521,7 +569,7 @@ txt_infoll_to_array(gal_data_t *datall, size_t *numdata)
           /* Put all the information from 'data' into the respective part
              of the array. About the pointers, instead of having to
              allocate them again, we will just set them to NULL so
-             'gal_data_free' doesn't remove them.*/
+             'gal_data_free' doesn't remove them. */
           dataarr[ind].flag       = data->flag;    data->flag=0;
           dataarr[ind].name       = data->name;    data->name=NULL;
           dataarr[ind].unit       = data->unit;    data->unit=NULL;
@@ -560,12 +608,12 @@ txt_get_info_line(char *line, gal_data_t **datall, char *comm_start,
 
   switch( gal_txt_line_stat(line) )
     {
-      /* Line is a comment, see if it has formatted information. */
+    /* Line is a comment, see if it has formatted information. */
     case GAL_TXT_LINESTAT_COMMENT:
       txt_info_from_comment(line, datall, comm_start, inplace);
       break;
 
-      /* Line is actual data, use it to fill in the gaps.  */
+    /* Line is actual data, use it to fill in the gaps.  */
     case GAL_TXT_LINESTAT_DATAROW:
       ++dsize[0];
       if(*firstlinedone==0)
@@ -644,8 +692,8 @@ txt_get_info(char *filename, gal_list_str_t *lines, int format,
          lines, but also confirm/complete the info by parsing the first
          uncommented line. */
       while( getline(&line, &linelen, fp) != -1 )
-        txt_get_info_line(line, &datall, comm_start, &firstlinedone, format,
-                          dsize, 1);
+        txt_get_info_line(line, &datall, comm_start, &firstlinedone,
+                          format, dsize, 1);
 
 
       /* Clean up and close the file. */
@@ -735,7 +783,7 @@ txt_read_token(gal_data_t *data, gal_data_t *info, char *token,
   double      *d = data->array,    *db;
 
   /* See if this token is blank. */
-  int isblankstr = ( info->flag==GAL_TABLEINTERN_FLAG_ARRAY_IS_BLANK_STRING
+  int isblankstr = ( info->flag & GAL_TABLEINTERN_FLAG_ARRAY_IS_BLANK_STRING
                      ? ( strcmp(info->array,token)==0 ? 1 : 0 )
                      : 0);
 
@@ -877,14 +925,22 @@ txt_read_token(gal_data_t *data, gal_data_t *info, char *token,
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "column %zu "
                           "('%s') couldn't be read as a '%s' number.\n\n"
                           "If it was meant to be celestial coordinates (RA "
-                          "or Dec), please use the '_h_m_s' format for RA "
-                          "or '_d_m_s' for Dec. The '_:_:_' format is "
+                          "or Dec), please use the '_h_m_' format for RA "
+                          "or '_d_m_' for Dec. The '_:_:_' format is "
                           "ambiguous (can be used for both RA and Dec). "
                           "Alternatively, you can use the column arithmetic "
                           "operators 'ra-to-degree' or 'dec-to-degree' of "
                           "'asttable' which also accept the '_:_:_' "
-                          "format. For more, please run this command\n\n"
-                          "   $ info gnuastro \"column arithmetic\"",
+                          "format. However, the 'ra-to-degree' or "
+                          "'dec-to-degree' operators require the column "
+                          "to be identified as a string with metadata. "
+                          "Please run the command below to learn more "
+                          "about column metadata and columns with string "
+                          "contents (it is easier to just use the '_h_m_' "
+                          "or '_d_m_' formats which will be automatically "
+                          "converted to degrees without any operators or "
+                          "metadata):\n\n"
+                          "   $ info gnuastro \"Gnuastro text table format\"",
                           colnum, token,
                           gal_type_name(data->type, 1) );
           else
@@ -906,7 +962,7 @@ txt_fill(char *in_line, char **tokens, size_t maxcolnum,
 {
   gal_data_t *data;
   int notenoughcols=0;
-  size_t i, n=0, strwidth;
+  size_t i, len, n=0, strwidth;
   char *end, *line, *tmpstr, *aline=NULL;
 
   /* Make a copy of the input line if necessary. */
@@ -949,16 +1005,21 @@ txt_fill(char *in_line, char **tokens, size_t maxcolnum,
              copy the necessary number of characters into the 'tmpstr'
              string. We need to allocate this because the string column may
              be immediately (next character) followed by the next
-             column. This leaves us no space to put the '\0' character. */
+             column. This leaves us no space to copy the '\0'
+             character. Therefore we will add '\0' after 'strncpy'.
+
+             Also, it may happen (for example with plain-text editors that
+             remove trailing white space) that the width defined for the
+             last column becomes larger than the actual length of the
+             line. We should therefore first check how many characters we
+             should actually copy (may be less than 'disp_width'). See
+             https://savannah.gnu.org/bugs/index.php?62720 */
           strwidth=colinfo[n-1].disp_width;
-          errno=0;
-          tmpstr=malloc(strwidth+1);
-          if(tmpstr==NULL)
-            error(EXIT_FAILURE, errno, "%s: %zu bytes couldn't be allocated "
-                  "for variable 'tmpstr'", __func__, strwidth+1);
-          if(line+strwidth<end) strncpy(tmpstr, line, strwidth);
-          else                  strncpy(tmpstr, line, end-line);
-          tmpstr[strwidth]='\0';
+          len = (line+strwidth)<end ? strwidth : end-line;
+          tmpstr=gal_pointer_allocate(GAL_TYPE_UINT8, len+1, 0,
+                                      __func__, "tmpstr");
+          strncpy(tmpstr, line, len);
+          tmpstr[len]='\0';
           tokens[n]=tmpstr;
 
           /* Increment the line pointer beyond to the next token.*/
@@ -1087,6 +1148,7 @@ txt_read(char *filename, gal_list_str_t *lines, size_t *dsize,
               out->dsize=out->array=NULL;
             }
         }
+      gal_list_data_reverse(&out);
       break;
 
 
@@ -1206,12 +1268,16 @@ gal_txt_image_read(char *filename, gal_list_str_t *lines, size_t minmapsize,
 static int
 txt_stdin_has_contents(long timeout_microsec)
 {
+  int sout;
   fd_set fds;
   struct timeval tv;
 
-  /* Set the timeout time. */
-  tv.tv_sec  = 0;
-  tv.tv_usec = timeout_microsec;
+  /* Set the timeout time. We need to put the number of seconds in 'tv_sec'
+     and the remaining microseconds in 'tv_usec' (this cannot be larger
+     than one million, otherwise 'select' is going to abort with an
+     error). */
+  tv.tv_sec  = timeout_microsec/1000000;
+  tv.tv_usec = timeout_microsec%1000000;
 
   /* Initialize 'fd_set'. */
   FD_ZERO(&fds);
@@ -1219,14 +1285,29 @@ txt_stdin_has_contents(long timeout_microsec)
   /* Set standard input (STDIN_FILENO is 0) as the FD that must be read. */
   FD_SET(STDIN_FILENO, &fds);
 
-  /* 'select' takes the last file descriptor value + 1 in the fdset to
-     check, the fdset for reads, writes, and errors.  We are only passing
-     in reads.  the last parameter is the timeout.  select will return if
-     an FD is ready or the timeout has occurred. */
-  select(STDIN_FILENO+1, &fds, NULL, NULL, &tv);
+  /* From Glibc: "The 'select' function blocks the calling process until
+     there is activity on any of the specified sets of file descriptors, or
+     until the timeout period has expired". 'select' takes the last file
+     descriptor value+1 in the 'FD_SET' above as first argument. If the
+     second (reading), third (writing) and fourth (exception) are not NULL,
+     then it will check for the respective property(s).
 
-  // return 0 if STDIN is not ready to be read.
-  return FD_ISSET(STDIN_FILENO, &fds);
+     When successful (file descriptor has input for the desired action),
+     'select' will return 1. When the timeout has been reached, it will
+     return 0 and when there was an error it will return -1. If there is an
+     error, we'll abort the program and ask the user to contact us (its a
+     bug).*/
+  errno=0;
+  sout=select(STDIN_FILENO+1, &fds, NULL, NULL, &tv);
+  if(sout==-1)
+    error(EXIT_FAILURE, errno, "%s: a bug! Please contact us at '%s' "
+          "to fix the problem. The 'select' function has detected an "
+          "error", __func__, PACKAGE_BUGREPORT);
+
+  /* By this point, 'sout' only has a value of 1 (stdin is ready for
+     reading) or 0 (timeout was reached with no change, so it should't be
+     used). So simply return the value of 'sout'. */
+  return sout;
 }
 
 
@@ -1240,7 +1321,7 @@ gal_txt_stdin_read(long timeout_microsec)
   gal_list_str_t *out=NULL;
   size_t lineno=0, linelen=10;/* 'linelen' will be increased by 'getline'. */
 
-  /* If there is nothing  */
+  /* Only continue if standard input has any contents. */
   if( txt_stdin_has_contents(timeout_microsec) )
     {
       /* Allocate the space necessary to keep a copy of each line as we
@@ -1314,7 +1395,6 @@ make_fmts_for_printf(gal_data_t *datall, int leftadjust, size_t *len)
   size_t i=0, num=0;
   char fmt[2], lng[3];
 
-
   /* Allocate space for the output. */
   for(data=datall;data!=NULL;data=data->next) ++num;
   errno=0;
@@ -1323,11 +1403,9 @@ make_fmts_for_printf(gal_data_t *datall, int leftadjust, size_t *len)
     error(EXIT_FAILURE, errno, "%s: %zu bytes for fmts",
           __func__, FMTS_COLS*num*sizeof *fmts);
 
-
   /* Initialize the length to 0. */
   *len=0;
 
-
   /* Go over all the columns and make their formats. */
   for(data=datall;data!=NULL;data=data->next)
     {
@@ -1347,7 +1425,6 @@ make_fmts_for_printf(gal_data_t *datall, int leftadjust, size_t *len)
                                 ? gal_blank_as_string(data->type, 0)
                                 : NULL );
 
-
       /* Fill in the printing paramters. */
       gal_tableintern_col_print_info(data, GAL_TABLE_FORMAT_TXT, fmt, lng);
 
@@ -1363,15 +1440,25 @@ make_fmts_for_printf(gal_data_t *datall, int leftadjust, size_t *len)
          the final length of the overall format statement. The space in the
          end of 'fmts[i*2]' is to ensure that the columns don't merge, even
          if the printed string is larger than the expected width. */
-      if(data->disp_precision > 0)
-        *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%%s%d.%d%s%s ",
-                            leftadjust ? "-" : "", data->disp_width,
-                            data->disp_precision, lng, fmt);
-      else
-        *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%%s%d%s%s ",
-                            leftadjust ? "-" : "", data->disp_width,
-                            lng, fmt);
-
+      if(data->next)
+        {
+          if(data->disp_precision > 0)
+            *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%%s%d.%d%s%s ",
+                                leftadjust ? "-" : "", data->disp_width,
+                                data->disp_precision, lng, fmt);
+          else
+            *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%%s%d%s%s ",
+                                leftadjust ? "-" : "", data->disp_width,
+                                lng, fmt);
+        }
+      else /* Last column: no empty characters (no width or adjustment). */
+        {
+          if(data->disp_precision > 0)
+            *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%.%d%s%s",
+                                data->disp_precision, lng, fmt);
+          else
+            *len += 1 + sprintf(fmts[i*FMTS_COLS], "%%%s%s", lng, fmt);
+        }
 
       /* Set the string for the Gnuastro type. For strings, we also need to
          write the maximum number of characters.*/
@@ -1381,7 +1468,6 @@ make_fmts_for_printf(gal_data_t *datall, int leftadjust, size_t *len)
       else
         strcpy(fmts[i*FMTS_COLS+1], gal_type_name(data->type, 0));
 
-
       /* Increment the column counter. */
       ++i;
     }
@@ -1611,9 +1697,9 @@ gal_txt_write(gal_data_t *input, struct gal_fits_list_key_t **keylist,
       /* Check if the dimensionality and size is the same for all the
          elements. */
       if( input!=data && gal_dimension_is_different(input, data) )
-        error(EXIT_FAILURE, 0, "%s: the input list of datasets must have the "
-              "same sizes (dimensions and length along each dimension)",
-              __func__);
+        error(EXIT_FAILURE, 0, "%s: the input list of datasets must "
+              "have the same sizes (dimensions and length along each "
+              "dimension)", __func__);
     }
 
 
@@ -1628,9 +1714,10 @@ gal_txt_write(gal_data_t *input, struct gal_fits_list_key_t **keylist,
     {
       /* Make sure the file doesn't already exist. */
       if( gal_checkset_check_file_return(filename) )
-        error(EXIT_FAILURE, 0, "%s: %s already exists. For safety, this "
-              "function will not over-write an existing file. Please delete "
-              "it before calling this function", __func__, filename);
+        error(EXIT_FAILURE, 0, "%s: %s already exists. For safety, "
+              "this function will not over-write an existing file. "
+              "Please delete it before calling this function",
+              __func__, filename);
 
       /* Open the output file. */
       errno=0;
@@ -1657,26 +1744,14 @@ gal_txt_write(gal_data_t *input, struct gal_fits_list_key_t **keylist,
   switch(input->ndim)
     {
     case 1:
-      /* When the dataset is bring printed on standard output and its a
-         single number, don't print the column structure, because it will
-         add white-space characters which can be annoying when used in an
-         automatic script. */
-      if(fp==stdout && input->size==1 && input->next==NULL)
-        fprintf(fp, "%s\n",
-                gal_type_to_string(input->array, input->type, 0));
-
-      /* Dataset has more than one row AND more than one column, so follow
-         the basic text formatting (like extra white space to keep the
-         columns under each other). */
-      else
-        for(i=0;i<input->size;++i)                        /* Row.    */
-          {
-            j=0;
-            for(data=input;data!=NULL;data=data->next)    /* Column. */
-              txt_print_value(fp, data->array, data->type, i,
+      for(i=0;i<input->size;++i)                        /* Row.    */
+        {
+          j=0;
+          for(data=input;data!=NULL;data=data->next)    /* Column. */
+            txt_print_value(fp, data->array, data->type, i,
                             fmts[j++ * FMTS_COLS]);
-            fprintf(fp, "\n");
-          }
+          fprintf(fp, "\n");
+        }
       break;
 
 
